<?php
/**
 * @package Nicepage Website Builder
 * @author Nicepage https://www.nicepage.com
 * @copyright Copyright (c) 2016 - 2019 Nicepage
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPLv2 or later
 */
defined('_JEXEC') or die;

/**
 * Class NicepageHelpersNicepage
 */
class NicepageHelpersNicepage
{
    /**
     * Extension type name
     *
     * @var string
     */
    public static $extension = 'com_nicepage';

    /**
     * Get actions for nicepage component
     *
     * @return JObject
     */
    public static function getActions()
    {
        $user = JFactory::getUser();
        $result = new JObject;

        $assetName = 'com_nicepage';
        $level = 'component';

        $actions = JAccess::getActions('com_nicepage', $level);

        foreach ($actions as $action) {
            $result->set($action->name, $user->authorise($action->name, $assetName));
        }

        return $result;
    }

    /**
     * Get domain get parameter
     *
     * @return mixed|string
     */
    public static function getDomain()
    {
        $default = defined('NICEPAGE_DOMAIN') ? NICEPAGE_DOMAIN : '';
        $app = JFactory::getApplication();
        $domain = urldecode($app->input->getVar('domain', $default));
        if ($domain) {
            $domain = preg_replace('#^https?:#', '', $domain); // remove protocol
            $domain = preg_replace('#\/$#', '', $domain); // remove last slash
        }
        return $domain;
    }

    /**
     * Get site url
     *
     * @return string
     */
    public static function getSiteUrl()
    {
        return dirname(dirname((JURI::current())));
    }

    /**
     * Get files for nicepage starting
     *
     * @return array
     */
    public static function getStartFiles()
    {
        $domain = self::getDomain();

        $extension = JTable::getInstance('extension');
        $id = $extension->find(array('element' => 'com_nicepage'));
        $extension->load($id);
        $componentInfo = json_decode($extension->manifest_cache, true);
        $hash = $componentInfo['version'];

        return array(
            'editor' => self::getSiteUrl() . '/administrator/components/com_nicepage/assets/js/editor.js?ver=' . $hash,
            'loader' => $domain ? $domain . '/Editor/loader.js' : self::getLoader($hash),
            'manifest' => self::getSiteUrl() . '/administrator/components/com_nicepage/assets/app/app.manifest',
            'auth' => self::getSiteUrl() . '/administrator/components/com_nicepage/helpers/auth.php?uid=' . JFactory::getUser()->id . '&ver=' . $hash
        );
    }

    /**
     * Get custom loader file
     *
     * @param string $hash Hash for get parameter
     *
     * @return string
     */
    public static function getLoader($hash) {
        return self::getSiteUrl() . '/administrator/components/com_nicepage/assets/app/loader.js?ver=' . $hash;
    }

    /**
     * Get actions list for nicepage app
     *
     * @return array
     */
    public static function getEditorSettings()
    {
        $current = dirname(dirname((JURI::current())));
        $index = $current . '/administrator/index.php?option=com_nicepage&task=actions.';
        return array(
            'actions' => array(
                'uploadImage' => $index . 'uploadImage',
                'savePage' => $index . 'savePage',
                'getSite' => $index . 'getSite',
                'getSitePosts' => $index . 'getSitePosts',
                'getPage' => $index . 'getPage',
                'getManifest' => $index . 'getManifest',
                'saveSiteSettings' => $index . 'saveSiteSettings',
            ),
            'uploadImageOptions' => array(
                'formFileName' => 'async-upload'
            ),
            'dashboardUrl' => $current . '/administrator/',
            'editPostUrl' => $current . '/administrator/index.php?option=com_content&view=article&layout=edit&id={id}'
        );
    }

    /**
     * Get cms custom settings
     *
     * @return array
     */
    public static function getCmsSettings()
    {
        return array(
            'defaultImageUrl' => dirname(dirname((JURI::current()))) . '/administrator/components/com_nicepage/assets/images/nicepage-images/default-image.jpg',
            'isFirstStart' => false
        );
    }

    /**
     * Build site style css
     *
     * @param array  $params         plugin config parameters
     * @param string $pageCssUsedIds used css ids
     * @param string $html           public html of page
     * @param string $pageId         id of page
     *
     * @return string
     */
    public static function buildSiteStyleCss($params, $pageCssUsedIds, $html, $pageId)
    {
        $result = isset($params['siteStyleCss']) ? $params['siteStyleCss'] : '';
        $partsJson = isset($params['siteStyleCssParts']) ? $params['siteStyleCssParts'] : '';
        if ($partsJson) {
            $cssParts = json_decode($partsJson, true);
            if (!$pageCssUsedIds) {
                $usedIds = self::processUsedColors($cssParts, $html);
                self::updateUsedColor($usedIds, $pageId);
            } else {
                $usedIds = json_decode($pageCssUsedIds, true);
            }
            $headerFooterCssUsedIds = isset($params['headerFooterCssUsedIds']) ? json_decode($params['headerFooterCssUsedIds'], true) : [];
            $result   = '';
            foreach ($cssParts as $part) {
                if ($part['type'] !== 'color' || !empty($usedIds[$part['id']]) || !empty($headerFooterCssUsedIds[$part['id']])) {
                    $result .= $part['css'];
                }
            }
        }
        return $result;
    }

    /**
     * @param string $styles              common styles
     * @param string $publishHtml         publish html of page
     * @param string $publishHeaderFooter publish html of page
     *
     * @return array
     */
    public static function processAllColors($styles, $publishHtml, $publishHeaderFooter = '') {
        $split = preg_split('#(\/\*begin-color [^*]+\*\/[\s\S]*?\/\*end-color [^*]+\*\/)#', $styles, -1, PREG_SPLIT_DELIM_CAPTURE);
        $parts = array();
        foreach ($split as $part) {
            $part = trim($part);
            if (!$part) {
                continue;
            }

            if (preg_match('#\/\*begin-color ([^*]+)\*\/#', $part, $m)) {
                $id = 'color_' . $m[1];
                $parts[] = array(
                    'type' => 'color',
                    'id' => $id,
                    'css' => $part,
                );
                if (strpos($publishHtml, $m[1]) !== false) {
                    $usedIds[$id] = true;
                }
            } else {
                $parts[] = array(
                    'type' => '',
                    'css' => $part,
                );
            }
        }
        $headerFooterUsedIds = $publishHeaderFooter ? self::processUsedColors($parts, $publishHeaderFooter) : [];
        $usedIds = self::processUsedColors($parts, $publishHtml);
        return array(json_encode($parts), json_encode($usedIds), json_encode($headerFooterUsedIds));
    }

    /**
     * @param array  $parts       All styles array
     * @param string $publishHtml Html of page
     *
     * @return array
     */
    public static function processUsedColors($parts, $publishHtml)
    {
        $usedIds = array();
        foreach ($parts as &$part) {
            if (isset($part['id']) && strpos($publishHtml, preg_replace('#^color_#', '', $part['id'])) !== false) {
                $usedIds[$part['id']] = true;
            }
        }
        return $usedIds;
    }

    /**
     * Get nicepage properties
     *
     * @return mixed
     */
    public static function getConfig()
    {
        $ret = NicepageHelpersNicepage::getParamsTable()->getParameters();
        if (isset($ret['siteStyleCss']) && substr($ret['siteStyleCss'], 0, 6) === '<style') {
            // backward compatibility
            $css = preg_replace('#</?style[^>]*>#', '', $ret['siteStyleCss']);
            ob_start();
            include_once dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/assets/css/nicepage-dynamic.css';
            $dynamicPart = ob_get_clean();
            $ret['siteStyleCss'] = $css . $dynamicPart;
        }
        return $ret;
    }

    /**
     * Save nicepage settings
     *
     * @param array $data Data parameters
     *
     * @return mixed|string
     */
    public static function saveConfig($data)
    {
        $paramsTable = NicepageHelpersNicepage::getParamsTable();

        $params = $paramsTable->getParameters();
        $excludeParameters = array('option', 'action', 'controller', 'task', 'view');
        foreach ($data as $key => $value) {
            if (in_array($key, $excludeParameters)) {
                continue;
            }
            $params[$key] = $value;
        }
        $paramsTable->saveParameters($params);
    }

    /**
     * @param string $props Page properties
     *
     * @return mixed
     */
    public static function decodeProps($props)
    {
        return unserialize(call_user_func('base' . '64_decode', $props));
    }

    /**
     * @param array $props Page properties
     *
     * @return mixed
     */
    public static function encodeProps($props)
    {
        return call_user_func('base' . '64_encode', serialize($props));
    }

    /**
     * Get new sections object
     *
     * @return mixed
     */
    public static function getSectionsTable()
    {
        JLoader::register('PagesTableSections', dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/tables/sections.php');
        return JTable::getInstance('Sections', 'PagesTable');
    }

    /**
     * Get new params object
     *
     * @return mixed
     */
    public static function getParamsTable()
    {
        JLoader::register('PagesTableParams', dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/tables/params.php');
        return JTable::getInstance('Params', 'PagesTable');
    }

    /**
     * Clear preview page
     *
     * @param object $page sections page object
     *
     * @return null
     */
    public static function clearPreview($page) {
        $props = NicepageHelpersNicepage::decodeProps($page->props);
        $props['preview'] = '';
        $page->save(array('props' => NicepageHelpersNicepage::encodeProps($props)));
    }

    /**
     * @param array $parts  Used color list
     * @param int   $pageId Id of page
     */
    public static function updateUsedColor($parts, $pageId) {
        $page = NicepageHelpersNicepage::getSectionsTable();
        if ($page->load(array('page_id' => $pageId))) {
            $props = NicepageHelpersNicepage::decodeProps($page->props);
            $props['pageCssUsedIds'] = json_encode($parts);
            $page->save(array('props' => NicepageHelpersNicepage::encodeProps($props)));
        }
    }

    /**
     * @param object $sectionsObj sections page object
     *
     * @return mixed|string
     */
    public static function getSectionsScreenshots($sectionsObj)
    {
        if (!$sectionsObj) {
            return '';
        }

        $props = NicepageHelpersNicepage::decodeProps($sectionsObj->props);
        $publishHtml = isset($props['publishHtml']) ? $props['publishHtml'] : '';
        $bodyClass = isset($props['bodyClass']) ? $props['bodyClass'] : '';
        $bodyStyle = isset($props['bodyStyle']) ? $props['bodyStyle'] : '';
        $head = isset($props['head']) ? $props['head'] : '';
        $fonts = isset($props['fonts']) ? $props['fonts'] : '';
        $pageCssUsedIds = isset($props['pageCssUsedIds']) ? $props['pageCssUsedIds'] : '';
        $publishHtml = self::processSectionsHtml($publishHtml, false);
        preg_match_all('/<section[\s\S]+?<\/section>/', $publishHtml, $matches, PREG_SET_ORDER);
        $count = count($matches);
        if ($count > 4) {
            for ($i = 4; $i < $count; $i++) {
                $publishHtml = str_replace($matches[$i], '', $publishHtml);
            }
        }
        $assets = '/administrator/components/com_nicepage/assets';
        $nicepageCss = JURI::root(true) . $assets. '/css/nicepage.css';

        include_once  dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/helpers/nicepage.php';
        $params = NicepageHelpersNicepage::getConfig();
        $siteStyleCss = NicepageHelpersNicepage::buildSiteStyleCss($params, $pageCssUsedIds, $publishHtml, $sectionsObj->page_id);

        $ret = <<<EOF
<!DOCTYPE html>
<html>        
    <head>
    <style>
        body {
            cursor: pointer;
        }
    </style>
    <link rel="stylesheet" href="$nicepageCss">
    $fonts
    <style>
    	$siteStyleCss
        $head
    </style>
    </head>
    <body class="$bodyClass" style="$bodyStyle">
        $publishHtml
    </body>
</html>
EOF;
        return $ret;
    }

    /**
     * @param string $content  Page content
     * @param bool   $isPublic Flag for public content
     *
     * @return mixed
     */
    public static function processSectionsHtml($content, $isPublic = true) {
        $content = self::processDefaultImage($content);
        if ($isPublic) {
            $content = self::processForm($content);
            $content = self::processCustomPhp($content);
            $content = self::processControls($content);
        }
        $content = self::processPositions($content);
        return $content;
    }

    public static $controlName = '';
    /**
     * Process all custom controls on the header
     *
     * @param string $content content
     *
     * @return mixed
     */
    public static function processControls($content) {
        $controls = ['menu', 'logo', 'headline', 'search'];
        foreach ($controls as $value) {
            self::$controlName = $value;
            $content =  preg_replace_callback(
                '/<\!--np_' . $value . '--><!--np_json-->([\s\S]+?)<\!--\/np_json-->([\s\S]*?)<\!--\/np_' . $value . '-->/',
                function ($matches) {
                    $controlProps = json_decode(trim($matches[1]), true);
                    $controlTemplate = $matches[2];
                    ob_start();
                    include JPATH_ADMINISTRATOR . '/components/com_nicepage/views/controls/'. self::$controlName . '/' . self::$controlName . '.php';
                    return ob_get_clean();
                },
                $content
            );
        }
        return $content;
    }

    /**
     * Process all custom php on the page
     *
     * @param string $content Page content
     *
     * @return mixed
     */
    public static function processCustomPhp($content) {
        $content = preg_replace('/data-custom-php=["\']+(<\!--custom_php-->[\s\S]+?<\!--\/custom_php-->)["\']+([^>]*?>)/', '$2$1', $content);
        $content =  preg_replace_callback(
            '/<\!--custom_php-->([\s\S]+?)<\!--\/custom_php-->/',
            function ($matches) {
                $code = trim($matches[1]);
                $code = str_replace("<?php", "", $code);
                $code = str_replace("?>", "", $code);
                $code = str_replace("&quot;", "\"", $code);
                ob_start();
                @eval($code);
                return ob_get_clean();
            },
            $content
        );
        return $content;
    }

    /**
     * @var array All blocks on the page
     */
    public static $blockLayouts = array();

    /**
     * Process all positions on the page
     *
     * @param string $content Page content
     *
     * @return mixed
     */
    public static function processPositions($content) {
        $content = preg_replace_callback(
            '/<\!--position-->([\s\S]+?)<\!--\/position-->/',
            function ($match) {
                $block = $match[1];
                preg_match('/data-position="([^"]*)"/', $block, $match2);
                $position = $match2[1];
                $i = count(self::$blockLayouts) + 1;
                self::$blockLayouts[$i] = $block;
                $document = JFactory::getDocument();
                if ($position && $document->countModules($position) !== 0) {
                    $attr = array(
                        'style' => 'upstylefromplugin',
                        'iterator' => $i,
                        'title' => 'name-' . $i,
                        'id' => $i,
                        'name' => $position
                    );
                    include_once dirname(dirname(__FILE__)) . '/helpers/modules.php';
                    return $document->getBuffer('modules', $position, $attr);
                } else {
                    return '<div class="hidden-position" style="display: none"></div>';
                }
            },
            $content
        );
        return $content;
    }

    /**
     * Processing of default image path
     *
     * @param string $content Page content
     *
     * @return mixed
     */
    public static function processDefaultImage($content)
    {
        // replace default image placeholder
        $url = JURI::current() . 'administrator/components/com_nicepage/assets/images/nicepage-images/default-image.jpg';
        return str_replace('[image_default]', $url, $content);
    }

    /**
     * Processing of forms
     *
     * @param string $content Page content
     *
     * @return mixed
     */
    public static function processForm($content)
    {
        $content = preg_replace('/(<form[^>]*action=[\'\"]+).*?([\'\"][^>]*source=[\'\"]joomla)/', '$1index.php?option=com_nicepage&task=sendmail$2', $content);
        $content = preg_replace_callback(
            '/<form[^>]+redirect-address=[\'\"](.*?)[\'\"][^>]*>/',
            function ($matches) {
                if (preg_match('/<form[^>]+redirect=[\'\"]true[\'\"]/', $matches[0])) {
                    return $matches[0] . '<input type="hidden" name="redirect" value="' . $matches[1] . '">';
                }
                return $matches[0];
            },
            $content
        );
        return $content;
    }
}